package com.emato.cus.supervise.biz.acqInventoryInfo;

import com.emato.cus.supervise.biz.CusData;
import com.emato.cus.supervise.constant.EmtRecordTimeEnum;
import com.emato.cus.supervise.domainCus.CusAcqGoodsMove05;
import com.emato.cus.supervise.domainCus.CusAcqInventoryInfo03;
import com.emato.cus.supervise.domainOms.OmsCusUnitCode;
import com.emato.cus.supervise.domainOms.OmsMerchInfo;
import com.emato.cus.supervise.mapperOms.OmsCusUnitCodeMapper;
import com.emato.cus.supervise.service.cus.CusAcqInventoryInfoService03;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.beans.Transient;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author zengjunlin
 * @version 1.0
 * 2018-02-02 09:44
 */
@Component
public class AcqInventoryInfoCusData implements CusData {

    private static final Logger logger = LoggerFactory.getLogger(AcqInventoryInfoCusData.class);

    @Autowired
    private CusAcqInventoryInfoService03 cusAcqInventoryInfoService03;
    @Autowired
    private OmsCusUnitCodeMapper omsCusUnitCodeMapper;

    private static final String DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.I_ACQ_INVENTORY_INFO.getCode() + "]" + EmtRecordTimeEnum.I_ACQ_INVENTORY_INFO.getName();

    /**
     * 海关抓取库存信息为全量抓取
     * 先删除，再插入
     *
     * @param list
     * @return
     */
    public int writeCusWms(List<?> list) {
        logger.info("---------- 货物库存数据::写入本次监控数据开始 --------------------");

        // bug 20200914 修改为库存全量更新

        // 数据准备

        List<CusAcqInventoryInfo03> cusList = ( List<CusAcqInventoryInfo03>)list;
        // 全部删除，使用一个空集
        List<CusAcqInventoryInfo03> deleteList = Lists.newArrayList();
        // 新增库存list
        List<CusAcqInventoryInfo03> insertList = new ArrayList<>();

        // ------ 查询oms产品数据
        List<String> gUnitNameList = Lists.newArrayList();
        cusList.forEach(
                i -> {
                    gUnitNameList.add(i.getgUnit());
                });

        List<OmsCusUnitCode> codeList = omsCusUnitCodeMapper.getOmsCusUnitCodeList(gUnitNameList);
        Map<String,OmsCusUnitCode> omsCusUnitCodeMap = codeList.stream().collect(Collectors.toMap(k->k.getgUnitName(), v->v));

        for (CusAcqInventoryInfo03 info : cusList) {
            /*if (info.getgQty() == null) {
                logger.debug("{}为空，qty变为0.0", info.getSkuNo());
                info.setgQty(0.0);
            }else{
                logger.debug("{}不为空，qty: {}", info.getSkuNo(), info.getgQty());
            }*/

            OmsCusUnitCode omsProductInfo = omsCusUnitCodeMap.get(info.getgUnit());
            if(omsProductInfo != null) {
                info.setgUnit(omsProductInfo.getgUnitCode());
            }
            insertList.add(info);

        }

        int deleteResult = 0;
        int insertResult = 0;

        //先删除，再新增
        deleteResult = cusAcqInventoryInfoService03.deleteCusAcqInventoryInfo(deleteList);
        insertResult = cusAcqInventoryInfoService03.addCusAcqInventoryInfo(insertList);

        logger.info("---【" + DATA_TYPE_INFO + "】删除监控数据:{}条, 新增监控数据:{}条", deleteResult, insertResult);

        logger.info("---------- 货物库存数据::写入本次监控数据结束 --------------------");
        return deleteResult + insertResult;
    }

    /**
     * ！！！作废！！！
     * 海关抓取库存信息为全量抓取
     * 无则新增，有则更新
     *
     * @param list
     * @return
     */
    @Deprecated
    @Override
    public int writeCus(List<?> list) {
        logger.info("---------- 货物库存数据::写入本次监控数据开始 --------------------");

        List<CusAcqInventoryInfo03> cusList = ( List<CusAcqInventoryInfo03>)list;

        // 新增库存list
        List<CusAcqInventoryInfo03> insertList = new ArrayList<>();
        // 修改库存list
        List<CusAcqInventoryInfo03> updateList = new ArrayList<>();


        // ------ 查询现有wms_data库存信息表
        List<String> listSku = new ArrayList<>();


        List<String> gUnitNameList = Lists.newArrayList();
        cusList.forEach(
                i -> {
                    gUnitNameList.add(i.getgUnit());

                    // 查询条件
                    listSku.add(i.getSkuNo());
                });

        // bug 20200914 修改为库存全量更新

        // 查询已有库存信息表对应sku数据
        List<CusAcqInventoryInfo03> listInveInfo = cusAcqInventoryInfoService03.listCusAcqInventoryInfo(listSku);
        Map<String, CusAcqInventoryInfo03> exitInveInfo = new HashMap<>();
        // 会报异常："Duplicate key 99999"，即可能出现重复的id，
        //Map<String, String> exitInveInfo = listInveInfo.stream().collect(Collectors.toMap(k->k.getSkuNo(), v->v.getId()));
        //exitInveInfo = listInveInfo.stream().collect(Collectors.toMap(k -> k.getSkuNo() + "_" + k.getLocalEmsNo(), v -> v));
        /*List<String> exitInveInfo = new ArrayList<>();
        listInveInfo.stream().forEach(i->exitInveInfo.add(i.getSkuNo()));*/

        for (int i = 0, len = listInveInfo.size(); i < len; i++) {
            String key = listInveInfo.get(i).getSkuNo() + "_" + listInveInfo.get(i).getLocalEmsNo();
            if (exitInveInfo.containsKey(key)) {
                continue;
            } else {
                exitInveInfo.put(key, listInveInfo.get(i));
            }
        }

        List<OmsCusUnitCode> codeList = omsCusUnitCodeMapper.getOmsCusUnitCodeList(gUnitNameList);
        Map<String,OmsCusUnitCode> omsCusUnitCodeMap = codeList.stream().collect(Collectors.toMap(k->k.getgUnitName(), v->v));

        for (CusAcqInventoryInfo03 info:cusList) {
            if (info.getgQty() == null) {
                logger.debug("{}为空，qty变为0.0", info.getSkuNo());
                info.setgQty(0.0);
            }else{
                logger.debug("{}不为空，qty: {}", info.getSkuNo(), info.getgQty());
            }

            OmsCusUnitCode omsProductInfo = omsCusUnitCodeMap.get(info.getgUnit());
            if(omsProductInfo != null) {
                info.setgUnit(omsProductInfo.getgUnitCode());
            }

            if (exitInveInfo.containsKey(info.getSkuNo() + "_" + info.getLocalEmsNo())) {
                // 存在, 则更新
                updateList.add(info);
            } else {
                // 不存在, 则插入
                insertList.add(info);
            }

        }

        int insertResult = 0;
        int updateResult = 0;

        if (insertList.size() > 0) {
            insertResult = cusAcqInventoryInfoService03.addCusAcqInventoryInfo(insertList);
        }
        if (updateList.size() > 0) {
            updateResult = cusAcqInventoryInfoService03.updateCusAcqInventoryInfo(updateList);
        }


        logger.info("---【" + DATA_TYPE_INFO + "】写入监控数据:{}条, 更新监控数据:{}条", insertResult, updateResult);

        logger.info("---------- 货物库存数据::写入本次监控数据结束 --------------------");
        return insertResult + updateResult;
    }

}
