package com.emato.cus.supervise.biz.acqShelfLoad.up;

import com.emato.cus.supervise.constant.EmtRecordTimeEnum;
import com.emato.cus.supervise.domainOmsShop.OmsShopAcqShelfLoad;
import com.emato.cus.supervise.service.omsshop.OmsShopAcqShelfLoadService;
import com.google.common.collect.ImmutableMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

/**
 * 门店货物上下架数据 门店入场 即上架 出区
 * @author Scott Chen
 * @version 1.0
 * 2020-03-30 11:27
 */
@Component
public class AcqShelfLoadInUpOmsShopData {
    private static final Logger logger = LoggerFactory.getLogger(AcqShelfLoadInUpOmsShopData.class);

    @Autowired
    private OmsShopAcqShelfLoadService omsShopAcqShelfLoadService;

    private static final String DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_UP.getCode() + "]" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_UP.getName();


    public List getOmsShopData(String thisTime) {
        logger.info("---------- 门店入场数据::下架::获取本次监控数据开始 --------------------");

        if (StringUtils.isBlank(thisTime)) {
            String err = "---【" + DATA_TYPE_INFO + "】本次扫描时间为空.";
            logger.error(err);
            return Collections.EMPTY_LIST;
        }

        ImmutableMap param = ImmutableMap.of("thisTime", thisTime);
        List<OmsShopAcqShelfLoad> resultList = omsShopAcqShelfLoadService.getOmsShopAcqShelfLoadUp(param);

        if (resultList == null || resultList.size() == 0) {
            String err = "---【" + DATA_TYPE_INFO + "】获取本次监控数据为空.";
            logger.error(err);
            return Collections.EMPTY_LIST;
        }
        logger.info("---【" + DATA_TYPE_INFO + "】获取本次监控数据{}条.", resultList.size());

        logger.info("---------- 门店入场数据::下架::获取本次监控数据结束 --------------------");

        return resultList;
    }
}
